==> pvfs-1.5.1-pre5.tgz: pvfs-1.5.1-pre5/lib/pvfs_ftruncate64.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_ftruncate64_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_ftruncate64.c,v 1.3 2001/01/30 23:32:15 pcarns Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;
extern fdesc_p pfds[];
extern sockset socks;
extern jlist_p active_p;

#define PCNT pfds[fd]->fd.meta.p_stat.pcount
#define FINO pfds[fd]->fd.meta.u_stat.st_ino

int pvfs_ftruncate64(int fd, int64_t length)
{
	int i;
	ireq req;

	/* check for badness */
	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}  

	/* check for UNIX */
	if (!pfds[fd] || pfds[fd]->fs==FS_UNIX) {
		if ((off_t) length == length) {
			return(syscall(SYS_ftruncate, fd, (off_t) length));
		}
		else {
			errno = EINVAL;
			return(-1);
		}
	}

	if (pfds[fd]->fs == FS_PDIR) {
		errno = EISDIR;
		return(-1);
	}

	req.type                 = IOD_FTRUNCATE;
	req.majik_nr             = IOD_MAJIK_NR;
	req.dsize                = 0;
	req.req.ftruncate.f_ino  = FINO;
	req.req.ftruncate.cap    = pfds[fd]->fd.cap;
	req.req.ftruncate.length = length;

	/* build job to send reqs and recv acks */
	if (build_simple_jobs(pfds[fd], &req) < 0) {
		ERR("pvfs_ftruncate: build_simple_jobs failed\n");
		return(-1);
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("pvfs_ftruncate: do_jobs failed\n");
			return(-1);
		}
	}

	/* check acks from iods */
	for (i=0; i < PCNT; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			ERR1("pvfs_ftruncate: non-zero status returned from iod %d\n", i);
			errno = pfds[fd]->fd.iod[i].ack.eno;
			return(-1);
		}
	}
	return(0);
}

==> pvfs-1.5.2-pre1.tgz: pvfs-1.5.2-pre1/lib/pvfs_ftruncate64.c <==
/*
 * (C) 1995-2001 Clemson University and Argonne National Laboratory.
 *
 * See COPYING in top-level directory.
 */


static char pvfs_ftruncate64_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_ftruncate64.c,v 1.5 2001/05/01 00:41:16 rbross Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;
extern fdesc_p pfds[];
extern sockset socks;
extern jlist_p active_p;

#define PCNT pfds[fd]->fd.meta.p_stat.pcount
#define FINO pfds[fd]->fd.meta.u_stat.st_ino

int pvfs_ftruncate64(int fd, int64_t length)
{
	int i;
	ireq req;

	/* check for badness */
	if (fd < 0 || fd >= PVFS_NR_OPEN 
		 || (pfds[fd] && pfds[fd]->fs == FS_RESV)) 
	{
		errno = EBADF;
		return(-1);
	}  

	/* check for UNIX */
	if (!pfds[fd] || pfds[fd]->fs==FS_UNIX) {
#ifndef LARGE_FILE_SUPPORT
		if ((off_t) length == length) {
			return ftruncate(fd, (off_t) length);
		}
		else {
			errno = EINVAL;
			return(-1);
		}
#else
		return ftruncate64(fd, length);
#endif
	}

	if (pfds[fd]->fs == FS_PDIR) {
		errno = EISDIR;
		return(-1);
	}

	req.type                 = IOD_FTRUNCATE;
	req.majik_nr             = IOD_MAJIK_NR;
	req.dsize                = 0;
	req.req.ftruncate.f_ino  = FINO;
	req.req.ftruncate.cap    = pfds[fd]->fd.cap;
	req.req.ftruncate.length = length;

	/* build job to send reqs and recv acks */
	if (build_simple_jobs(pfds[fd], &req) < 0) {
		ERR("pvfs_ftruncate: build_simple_jobs failed\n");
		return(-1);
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("pvfs_ftruncate: do_jobs failed\n");
			return(-1);
		}
	}

	/* check acks from iods */
	for (i=0; i < PCNT; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			ERR1("pvfs_ftruncate: non-zero status returned from iod %d\n", i);
			errno = pfds[fd]->fd.iod[i].ack.eno;
			return(-1);
		}
	}
	return(0);
}

/*
 * Local variables:
 *  c-indent-level: 3
 *  c-basic-offset: 3
 *  tab-width: 3
 * End:
 */
